/*********************************************************************
** ATA LOW LEVEL I/O DRIVER -- ATAIOTRC.C
**
** by Hale Landis (landis@sugs.tware.com)
**
** There is no copyright and there are no restrictions on the use
** of this ATA Low Level I/O Driver code.  It is distributed to
** help other programmers understand how the ATA device interface
** works and it is distributed without any warranty.  Use this
** code at your own risk.
**
** This code is based on the ATA-2, ATA-3 and ATA-4 standards and
** on interviews with various AT controller and drive designers.
**
** This code has been run on many AT (IDE) drives and
** MFM/RLL controllers.  This code may be a little
** more picky about the status it sees at various times.  A real
** BIOS probably would not check the status as carefully.
**
** Compile with one of the Borland C or C++ compilers.
**
** This C source contains the low level I/O trace functions.
*********************************************************************/

#include <stdio.h>
#include <string.h>
#include <dos.h>

#include "ataio.h"

#include "ataiopd.h"

/**************************************************************/

/* trace dump buffer returned by trc_err_dump2() */
/* trc_cht_dump2() and trc_llt_dump2()           */

static unsigned char trcDmpBuf[100];

/* buffer used to assemble print lines */

static unsigned char prtBuf[64];

/**************************************************************/

/* command name lookup table and lookup function */

static struct
{
   unsigned char cmdCode;
   unsigned char * cmdName;
} cmdNames[] =
   {
      0xE5 ,   "CHECK POWER MODE" ,
      0x98 ,   "CHECK POWER MODE" ,
      0x08 ,   "DEVICE RESET" ,
      0x90 ,   "EXECUTE DEVICE DIAGNOSTIC" ,
      0x50 ,   "FORMAT TRACK" ,
      0xEC ,   "IDENTIFY DEVICE" ,
      0xA1 ,   "IDENTIFY PACKET DEVICE" ,
      0xE3 ,   "IDLE" ,
      0x97 ,   "IDLE" ,
      0xE1 ,   "IDLE IMMEDIATE" ,
      0x95 ,   "IDLE IMMEDIATE" ,
      0x91 ,   "INITIALIZE DRIVE PARAMETERS" ,
      0x00 ,   "NOP" ,
      0xA0 ,   "PACKET" ,
      0xE4 ,   "READ BUFFER" ,
      0xC4 ,   "READ MULTIPLE" ,
      0x20 ,   "READ SECTORS" ,
      0x40 ,   "READ VERIFY SECTORS" ,
      0x10 ,   "RECALIBRATE" ,
      0x70 ,   "SEEK" ,
      0xEF ,   "SET FEATURES" ,
      0xC6 ,   "SET MULTIPLE MODE" ,
      0xE6 ,   "SLEEP" ,
      0x99 ,   "SLEEP" ,
      0xE2 ,   "STANDBY" ,
      0x96 ,   "STANDBY" ,
      0xE0 ,   "STANDBY IMMEDIATE" ,
      0x94 ,   "STANDBY IMMEDIATE" ,
      0xE8 ,   "WRITE BUFFER" ,
      0xC5 ,   "WRITE MULTIPLE" ,
      0x30 ,   "WRITE SECTORS" ,
      0x3C ,   "WRITE VERIFY" ,
      0x00 ,   ""                   /* end of table */
   } ;

unsigned char * trc_get_cmd_name( unsigned char flg, unsigned char cc )

{
   int ndx = 0;

   if ( ( flg == TRC_FLAG_EMPTY ) || ( flg == TRC_FLAG_CDB ) )
      return "";
   if ( flg == TRC_FLAG_SRST )
      return "SOFT RESET";
   while ( * cmdNames[ndx].cmdName )
   {
      if ( cc == cmdNames[ndx].cmdCode )
         return cmdNames[ndx].cmdName;
      ndx ++ ;
   }
   return "";
}

/**************************************************************/

/* ATA status names lookup table and lookup function */

static struct
{
   unsigned char bitPos;
   unsigned char * bitName;
} ataStatusNames[] =
   {
      0x80 , "BSY "  ,
      0x40 , "DRDY " ,
      0x20 , "DF "   ,
      0x10 , "DSC "  ,
      0x08 , "DRQ "  ,
      0x04 , "CORR " ,
      0x02 , "IDX "  ,
      0x01 , "ERR "
   } ;

static unsigned char ataStatusNameBuf[48];

unsigned char * trc_get_st_bit_name( unsigned char st )

{
   int ndx;

   if ( st & 0x80 )
      st = 0x80;
   * ataStatusNameBuf = 0;
   for ( ndx = 0; ndx < 8; ndx ++ )
   {
      if ( st & ataStatusNames[ndx].bitPos )
         strcat( ataStatusNameBuf, ataStatusNames[ndx].bitName );
   }
   return ataStatusNameBuf;
}

/**************************************************************/

/* ATA error names lookup table and lookup function */

static struct
{
   unsigned char bitPos;
   unsigned char * bitName;
} ataErrorNames[] =
   {
      0x80 , "BBK "   ,
      0x40 , "UNC "   ,
      0x20 , "MC "    ,
      0x10 , "IDNF "  ,
      0x08 , "MCR "   ,
      0x04 , "ABRT "  ,
      0x02 , "TK0NF " ,
      0x01 , "AMNF "
   } ;

static unsigned char ataErrorNameBuf[48];

unsigned char * trc_get_er_bit_name( unsigned char er )

{
   int ndx;

   * ataErrorNameBuf = 0;
   for ( ndx = 0; ndx < 8; ndx ++ )
   {
      if ( er & ataErrorNames[ndx].bitPos )
         strcat( ataErrorNameBuf, ataErrorNames[ndx].bitName );
   }
   return ataErrorNameBuf;
}

/**************************************************************/

/* error name lookup table and lookup function */

static struct
{
   int errCode;
   unsigned char * errName;
} errNames[] =
   {
       1 ,  "Soft Reset timed out polling for device 0 to set BSY=0"  ,
       2 ,  "Soft Reset timed out polling device 1"                   ,
       3 ,  "Soft Reset timed out polling for device 1 to set BSY=0"  ,

      11 ,  "Device selection timed out polling for BSY=0"            ,
      12 ,  "Device selection timed out polling for RDY=1"            ,

      21 ,  "Non-Data command ended with bad status"                  ,
      22 ,  "Non-Data command timed out waiting for an interrupt"     ,
      23 ,  "Non-Data command timed out polling for BSY=0"            ,
      24 ,  "Exec Dev Diag command timed out polling device 1"        ,

      31 ,  "PIO Data In command terminated by error status"          ,
      32 ,  "Device should be ready to transfer data but DRQ=0"       ,
      33 ,  "PIO Data In command ended with bad status"               ,
      34 ,  "PIO Data In command timed out waiting for an interrupt"  ,
      35 ,  "PIO Data In command timed out polling for BSY=0"         ,

      41 ,  "PIO Data Out command terminated by error status"         ,
      42 ,  "Device should be ready to transfer data but DRQ=0"       ,
      43 ,  "PIO Data Out command ended with bad status"              ,
      44 ,  "PIO Data Out command timed out waiting for an interrupt" ,
      45 ,  "PIO Data Out command timed out polling for BSY=0"        ,
      46 ,  "Extra interrupt at start of a PIO Data Out command"      ,
      47 ,  "PIO Data Out command timed out polling for BSY=0"        ,

      51 ,  "Timeout waiting for BSY=0/DRQ=1 for cmd packet transfer" ,
      52 ,  "Bad status at command packet transfer time"              ,
      53 ,  "Timeout waiting for interrupt for data packet transfer"  ,
      54 ,  "Timeout polling for BSY=0/DRQ=1 for a data packet"       ,
      55 ,  "Bad status at data packet transfer time"                 ,
      56 ,  "Timout waiting for final interrupt at end of command"    ,
      57 ,  "Timeout polling for final BSY=0 at end of command"       ,
      58 ,  "Bad status at end of command"                            ,
      59 ,  "Buffer overrun (host buffer too small)"                  ,

      0  ,   ""                                             /* end of table */
   } ;

unsigned char * trc_get_err_name( int ec )

{
   int ndx = 0;

   while ( * errNames[ndx].errName )
   {
      if ( ec == errNames[ndx].errCode )
         return errNames[ndx].errName;
      ndx ++ ;
   }
   return "";
}

/**************************************************************/

/* command or reset error display data */

static int errDmpLine = 0;

/**************************************************************/

/* start the display of a command or reset error display */

void trc_err_dump1( void )

{

   errDmpLine = 1;
}

/**************************************************************/

/* return one line of a command or reset error display, */
/* returns NULL at end                                  */

unsigned char * trc_err_dump2( void )

{

   if ( reg_cmd_info.flg == TRC_FLAG_EMPTY )
      return "";
   if ( errDmpLine == 1 )
   {
      errDmpLine = 2;
      if ( reg_cmd_info.flg == TRC_FLAG_SRST )
         sprintf( trcDmpBuf, "  Command Code: -- %s ",
                             trc_get_cmd_name( TRC_FLAG_SRST, 0 ) );
      else
         sprintf( trcDmpBuf, "  Command Code: %02X = %s ",
                             reg_cmd_info.cmd,
                             trc_get_cmd_name( TRC_FLAG_CMD, reg_cmd_info.cmd ) );
      return trcDmpBuf;
   }
   if ( errDmpLine == 2 )
   {
      errDmpLine = 3;
      if ( reg_cmd_info.cmd == CMD_PACKET )
      {
         if ( reg_atapi_cp_size == 12 )
            sprintf( trcDmpBuf, "     ATAPI CDB: %02X%02X %02X%02X "
                                                "%02X%02X %02X%02X "
                                                "%02X%02X %02X%02X ",
                     reg_atapi_cp_data[0], reg_atapi_cp_data[1],
                     reg_atapi_cp_data[2], reg_atapi_cp_data[3],
                     reg_atapi_cp_data[4], reg_atapi_cp_data[5],
                     reg_atapi_cp_data[6], reg_atapi_cp_data[7],
                     reg_atapi_cp_data[8], reg_atapi_cp_data[9],
                     reg_atapi_cp_data[10], reg_atapi_cp_data[11] );
         else
            sprintf( trcDmpBuf, "     ATAPI CDB: %02X%02X %02X%02X "
                                                "%02X%02X %02X%02X "
                                                "%02X%02X %02X%02X "
                                                "%02X%02X %02X%02X ",
                     reg_atapi_cp_data[0], reg_atapi_cp_data[1],
                     reg_atapi_cp_data[2], reg_atapi_cp_data[3],
                     reg_atapi_cp_data[4], reg_atapi_cp_data[5],
                     reg_atapi_cp_data[6], reg_atapi_cp_data[7],
                     reg_atapi_cp_data[8], reg_atapi_cp_data[9],
                     reg_atapi_cp_data[10], reg_atapi_cp_data[11],
                     reg_atapi_cp_data[12], reg_atapi_cp_data[13],
                     reg_atapi_cp_data[14], reg_atapi_cp_data[15] );
         return trcDmpBuf;
      }
   }
   if ( errDmpLine == 3 )
   {
      errDmpLine = 4;
      sprintf( trcDmpBuf, "Driver ErrCode: %d %s ",
                          reg_cmd_info.ec, trc_get_err_name( reg_cmd_info.ec ) );
      return trcDmpBuf;
   }
   if ( errDmpLine == 4 )
   {
      errDmpLine = 5;
      if ( reg_cmd_info.to )
      {
         sprintf( trcDmpBuf, "                   "
                             "Driver timed out (use SHOWLL command for details) !" );
         return trcDmpBuf;
      }
   }
   if ( errDmpLine == 5 )
   {
      errDmpLine = 6;
      sprintf( trcDmpBuf, " Bytes xferred: %ld (%lXH) ",
                        reg_cmd_info.totalBytesXfer, reg_cmd_info.totalBytesXfer );
      return trcDmpBuf;
   }
   if ( errDmpLine == 6 )
   {
      errDmpLine = 7;
      sprintf( trcDmpBuf, " Device Status: %02X = %s ", reg_cmd_info.st2,
                        trc_get_st_bit_name( reg_cmd_info.st2 ) );
      return trcDmpBuf;
   }
   if ( errDmpLine == 7 )
   {
      errDmpLine = 8;
      sprintf( trcDmpBuf, "  Device Error: %02X = %s ", reg_cmd_info.er2,
                         trc_get_er_bit_name( reg_cmd_info.er2 ) );
      return trcDmpBuf;
   }
   if ( errDmpLine == 8 )
   {
      errDmpLine = 9;
      if ( reg_cmd_info.cmd == CMD_PACKET )
      {
         sprintf( trcDmpBuf, "  Failure bits: %04XH (ATAPI protocol errors)",
                            reg_cmd_info.failbits );
         return trcDmpBuf;
      }
   }
   if ( errDmpLine == 9 )
   {
      errDmpLine = 10;
      sprintf( trcDmpBuf, " ATA Intf Regs: FR  ER  SC  SN  CL  CH  DH  CM  ST  AS  DC " );
      return trcDmpBuf;
   }
   if ( errDmpLine == 10 )
   {
      errDmpLine = 11;
      if ( reg_cmd_info.flg == TRC_FLAG_SRST )
         sprintf( trcDmpBuf, "    Cmd Params: "
                  /* fr  er  sc  sn  cl  ch  dh  cm  st  as  dc */
                    "--  --  --  --  --  --  --  --  --  --  04 " );
      else
         sprintf( trcDmpBuf, "    Cmd Params: "
                  /*  fr   er   sc    sn    cl    ch    dh    cm   st  as   dc */
                    "%02X  --  %02X  %02X  %02X  %02X  %02X  %02X  --  --  %02X ",
                     reg_cmd_info.fr1, reg_cmd_info.sc1, reg_cmd_info.sn1,
                     reg_cmd_info.cl1, reg_cmd_info.ch1, reg_cmd_info.dh1,
                     reg_cmd_info.cmd, reg_cmd_info.dc1 );
      return trcDmpBuf;
   }
   if ( errDmpLine == 11 )
   {
      errDmpLine = 12;
      sprintf( trcDmpBuf, "     After Cmd: "
                  /* fr   er    sc    sn    cl    ch    dh   cm   st    as   dc */
                    "--  %02X  %02X  %02X  %02X  %02X  %02X  --  %02X  %02X  -- ",
                     reg_cmd_info.er2, reg_cmd_info.sc2, reg_cmd_info.sn2,
                     reg_cmd_info.cl2, reg_cmd_info.ch2, reg_cmd_info.dh2,
                     reg_cmd_info.st2, reg_cmd_info.as2 );
      return trcDmpBuf;
   }
   return NULL;
}

/**********************************************************/

/* command history trace buffer */

#define MAX_CHT 100

static struct
{
   unsigned char flg;         /* 0=empty, 1=soft reset, 2=command, 3=cdb */
   unsigned char cmd;         /* command code       or cdb size   */
   unsigned char fr1;         /* feature reg before or cdb byte 0 */
   unsigned char sc1;         /* sec cnt before     or cdb byte 1 */
   unsigned char sn1;         /* sec num before     or cdb continued */
   unsigned char cl1;         /* cyl low before */
   unsigned char ch1;         /* cyl high before */
   unsigned char dh1;         /* drive head before */
   unsigned char dc1;         /* device control before */
   unsigned char ec;          /* detailed error code */
   unsigned char to;          /* not zero if time out error */
   unsigned char st2;         /* status reg after */
   unsigned char as2;         /* alt status reg after */
   unsigned char er2;         /* error reg after */
   unsigned char sc2;         /* sec cnt after */
   unsigned char sn2;         /* sec num after */
   unsigned char cl2;         /* cyl low after */
   unsigned char ch2;         /* cyl high after */
   unsigned char dh2;         /* drive head after */
}  chtBuf[MAX_CHT];

static int chtCur = 0;
static int chtDmpLine = 0;
static int chtDmpNdx = 0;

/**************************************************************/

/* place an command or reset entry into */
/* the command history trace buffer     */

extern void trc_cht( void )

{

   chtBuf[chtCur].flg = reg_cmd_info.flg;
   chtBuf[chtCur].cmd = reg_cmd_info.cmd;
   chtBuf[chtCur].fr1 = reg_cmd_info.fr1;
   chtBuf[chtCur].sc1 = reg_cmd_info.sc1;
   chtBuf[chtCur].sn1 = reg_cmd_info.sn1;
   chtBuf[chtCur].cl1 = reg_cmd_info.cl1;
   chtBuf[chtCur].ch1 = reg_cmd_info.ch1;
   chtBuf[chtCur].dh1 = reg_cmd_info.dh1;
   chtBuf[chtCur].dc1 = reg_cmd_info.dc1;
   chtBuf[chtCur].ec  = reg_cmd_info.ec ;
   chtBuf[chtCur].to  = reg_cmd_info.to ;
   chtBuf[chtCur].st2 = reg_cmd_info.st2;
   chtBuf[chtCur].as2 = reg_cmd_info.as2;
   chtBuf[chtCur].er2 = reg_cmd_info.er2;
   chtBuf[chtCur].sc2 = reg_cmd_info.sc2;
   chtBuf[chtCur].sn2 = reg_cmd_info.sn2;
   chtBuf[chtCur].cl2 = reg_cmd_info.cl2;
   chtBuf[chtCur].ch2 = reg_cmd_info.ch2;
   chtBuf[chtCur].dh2 = reg_cmd_info.dh2;
   chtCur ++ ;
   if ( chtCur >= MAX_CHT )
      chtCur = 0;
}

/**************************************************************/

/* place an atapi command packet entry into */
/* the command history trace buffer         */

void trc_cht_pkt( void )

{
   int ndx;
   unsigned char * cp1;
   unsigned char * cp2;

   chtBuf[chtCur].flg = TRC_FLAG_CDB;
   chtBuf[chtCur].cmd = reg_atapi_cp_size;
   cp1 = reg_atapi_cp_data;
   cp2 = & chtBuf[chtCur].fr1;
   for ( ndx = 0; ndx < reg_atapi_cp_size; ndx ++ )
   {
      * cp2 = * cp1;
      cp2 ++ ;
      cp1 ++ ;
   }
   chtCur ++ ;
   if ( chtCur >= MAX_CHT )
      chtCur = 0;
}

/**************************************************************/

/* clear the command history trace buffer */

void trc_cht_dump0( void )

{

   for ( chtCur = 0; chtCur < MAX_CHT; chtCur ++ )
      chtBuf[chtCur].flg = TRC_FLAG_EMPTY;
   chtCur = 0;
}

/**************************************************************/

/* start a dump of the command history trace buffer */

void trc_cht_dump1( void )

{

   chtDmpLine = 0;
   chtDmpNdx = chtCur + 1;
   if ( chtDmpNdx >= MAX_CHT )
      chtDmpNdx = 0;
}

/**************************************************************/

/* return one line of the command history trace buffer, */
/* returns NULL at end.                                 */

unsigned char * trc_cht_dump2( void )

{
   unsigned long lba;
   unsigned char * cp;

   chtDmpLine ++ ;
   if ( chtDmpLine == 1 )     /* 1st line is 1st line of heading */
   {
                        /*0        1         2         3         4         5         6         7      7*/
                        /*12345678901234567890123456789012345678901234567890123456789012345678901234567*/
      strcpy( trcDmpBuf, "<--------- registers before ------> <---------- registers after ------> <--->" );
      return trcDmpBuf;
   }
   if ( chtDmpLine == 2 )     /* 2nd line is 2nd line of heading */
   {
                        /*0        1         2         3         4         5         6         7      7*/
                        /*12345678901234567890123456789012345678901234567890123456789012345678901234567*/
      strcpy( trcDmpBuf, "CM FR SC SN ChCl DH DC CHS--or--LBA ST AS ER SC SN ChCl DH CHS--or--LBA EC TO" );
      return trcDmpBuf;
   }
   while ( 1 )
   {
      if ( chtDmpNdx == chtCur )
         return NULL;
      if ( chtBuf[chtDmpNdx].flg != TRC_FLAG_EMPTY )
         break;
      chtDmpNdx ++ ;
      if ( chtDmpNdx >= MAX_CHT )
         chtDmpNdx = 0;
   }
   if ( chtBuf[chtDmpNdx].flg == TRC_FLAG_CDB )
   {
      cp = & chtBuf[chtDmpNdx].fr1;
      if ( chtBuf[chtDmpNdx].cmd == 12 )
         sprintf( trcDmpBuf, "   ATAPI CDB: %02X%02X %02X%02X "
                                           "%02X%02X %02X%02X "
                                           "%02X%02X %02X%02X ",
                  * ( cp + 0 ), * ( cp + 1 ),
                  * ( cp + 2 ), * ( cp + 3 ),
                  * ( cp + 4 ), * ( cp + 5 ),
                  * ( cp + 6 ), * ( cp + 7 ),
                  * ( cp + 8 ), * ( cp + 9 ),
                  * ( cp + 10 ), * ( cp + 11 ) );
      else
         sprintf( trcDmpBuf, "   ATAPI CDB: %02X%02X %02X%02X "
                                           "%02X%02X %02X%02X "
                                           "%02X%02X %02X%02X "
                                           "%02X%02X %02X%02X ",
                  * ( cp + 0 ), * ( cp + 1 ),
                  * ( cp + 2 ), * ( cp + 3 ),
                  * ( cp + 4 ), * ( cp + 5 ),
                  * ( cp + 6 ), * ( cp + 7 ),
                  * ( cp + 8 ), * ( cp + 9 ),
                  * ( cp + 10 ), * ( cp + 11 ),
                  * ( cp + 12 ), * ( cp + 13 ),
                  * ( cp + 14 ), * ( cp + 15 ) );
      chtDmpNdx ++ ;
      if ( chtDmpNdx >= MAX_CHT )
         chtDmpNdx = 0;
      return trcDmpBuf;
   }
   if ( chtBuf[chtDmpNdx].flg == TRC_FLAG_SRST )
   {
      strcpy( trcDmpBuf, ".. S o f t   R e s e t ............ " );
   }
   else
   {
      sprintf( trcDmpBuf,
               "%02X %02X %02X %02X %02X%02X %02X %02X ",
               chtBuf[chtDmpNdx].cmd, chtBuf[chtDmpNdx].fr1,
               chtBuf[chtDmpNdx].sc1, chtBuf[chtDmpNdx].sn1,
               chtBuf[chtDmpNdx].ch1, chtBuf[chtDmpNdx].cl1,
               chtBuf[chtDmpNdx].dh1, chtBuf[chtDmpNdx].dc1 );
      if ( chtBuf[chtDmpNdx].dh1 & 0x40 )
      {
         lba = chtBuf[chtDmpNdx].dh1 & 0x0f;
         lba = lba << 8;
         lba = lba | chtBuf[chtDmpNdx].ch1;
         lba = lba << 8;
         lba = lba | chtBuf[chtDmpNdx].cl1;
         lba = lba << 8;
         lba = lba | chtBuf[chtDmpNdx].sn1;
         sprintf( prtBuf, "   %9lu ", lba );
      }
      else
      {
         sprintf( prtBuf,
                  "%5u %2u %3u ",
                  (int) ( chtBuf[chtDmpNdx].ch1 << 8 ) | chtBuf[chtDmpNdx].cl1,
                  (int) chtBuf[chtDmpNdx].dh1 & 0x0f,
                  (int) chtBuf[chtDmpNdx].sn1 );
      }
      strcat( trcDmpBuf, prtBuf );
   }
   sprintf( prtBuf,
            "%02X %02X %02X %02X %02X %02X%02X %02X ",
            chtBuf[chtDmpNdx].st2, chtBuf[chtDmpNdx].as2,
            chtBuf[chtDmpNdx].er2, chtBuf[chtDmpNdx].sc2,
            chtBuf[chtDmpNdx].sn2, chtBuf[chtDmpNdx].ch2,
            chtBuf[chtDmpNdx].cl2, chtBuf[chtDmpNdx].dh2 );
   strcat( trcDmpBuf, prtBuf );
   if ( chtBuf[chtDmpNdx].dh2 & 0x40 )
   {
      lba = chtBuf[chtDmpNdx].dh2 & 0x0f;
      lba = lba << 8;
      lba = lba | chtBuf[chtDmpNdx].ch2;
      lba = lba << 8;
      lba = lba | chtBuf[chtDmpNdx].cl2;
      lba = lba << 8;
      lba = lba | chtBuf[chtDmpNdx].sn2;
      sprintf( prtBuf, "   %9lu ", lba );
   }
   else
   {
      sprintf( prtBuf,
               "%5u %2u %3u ",
               (int) ( chtBuf[chtDmpNdx].ch2 << 8 ) | chtBuf[chtDmpNdx].cl2,
               (int) chtBuf[chtDmpNdx].dh2 & 0x0f,
               (int) chtBuf[chtDmpNdx].sn2 );
   }
   strcat( trcDmpBuf, prtBuf );
   sprintf( prtBuf,
            "%2d %2d",
            chtBuf[chtDmpNdx].ec, chtBuf[chtDmpNdx].to );
   strcat( trcDmpBuf, prtBuf );
   chtDmpNdx ++ ;
   if ( chtDmpNdx >= MAX_CHT )
      chtDmpNdx = 0;
   return trcDmpBuf;
}

/**********************************************************/

/* Low-level trace buffer */

#define MAX_LLT 500

static struct
{
   unsigned char addr;
   unsigned char data;
   unsigned char type;
   unsigned char rep;
} lltBuf[MAX_LLT];

static int lltCur = 0;
static int lltDmpLine = 0;
static int lltDmpNdx = 0;

static unsigned char lltAddr = 0;
static unsigned char lltData = 0;
static unsigned char lltType = 0;
static unsigned long lltRep = 0L;

static struct
{
   unsigned char typeId;      /* trace entry type */
   unsigned char * typeNm;    /* trace entry name */
} type_nm[]
   =
   {
                     /*<opr> <--register---> <data - note>*/
      TRC_LLT_INB   , "INB   " ,
      TRC_LLT_OUTB  , "OUTB  " ,
      TRC_LLT_INW   , "INW   " ,
      TRC_LLT_OUTW  , "OUTW  " ,
      TRC_LLT_INSW  , "INSW  " ,
      TRC_LLT_OUTSW , "OUTSW " ,
      TRC_LLT_S_CFG , "===== Start Dev Cnfg  ",
      TRC_LLT_S_RST , "===== Start Reset     ",
      TRC_LLT_S_ND  , "===== Start ND cmd    ",
      TRC_LLT_S_PDI , "===== Start PDI cmd   ",
      TRC_LLT_S_PDO , "===== Start PDO cmd   ",
      TRC_LLT_S_PI  , "===== Start PI cmd    ",
      TRC_LLT_WINT  , "..... Wait for INTRQ  ",
      TRC_LLT_PNBSY , "..... Poll for BSY=0  ",
      TRC_LLT_PRDY  , "..... Poll for DRDY=1 ",
      TRC_LLT_TOUT  , "..... T I M E O U T   ",
      TRC_LLT_ERROR , "..... E R R O R       ",
      TRC_LLT_P_CMD , "..... ATAPI Cmd Code  ",
      TRC_LLT_DELAY , "..... DELAY ~100ms    ",
      TRC_LLT_DEBUG , "..... DEBUG           ",
      TRC_LLT_E_CFG , "===== End Dev Cnfg    ",
      TRC_LLT_E_RST , "===== End Reset       ",
      TRC_LLT_E_ND  , "===== End ND cmd      ",
      TRC_LLT_E_PDI , "===== End PDI cmd     ",
      TRC_LLT_E_PDO , "===== End PDO cmd     ",
      TRC_LLT_E_PI  , "===== End PI cmd      ",
      0             , "????? " ,
   } ;

static unsigned char * reg_nm[]  /* register names for trace */
   =
   {
  /*<opr> <--register---> <data - note>*/
         "Data            " , /* 0 data reg */
         "Error/Feature   " , /* 1 error & feature */
         "SectorCount     " , /* 2 sector count */
         "SectorNumber    " , /* 3 sector number */
         "CylinderLow     " , /* 4 cylinder low */
         "CylinderHigh    " , /* 5 cylinder high */
         "DriveHead       " , /* 6 drive/head */
         "Status/Cmd      " , /* 7 primary status & command */
         "AltStat/DevCtrl " , /* 8 alternate status & device control */
         "DevAddr         " , /* 9 device address */
   } ;

/*********************************************************/

/* place an entry into the low level trace buffer */

void trc_llt( unsigned char addr,
              unsigned char data,
              unsigned char type )

{

   if ( ( addr == lltAddr )
        &&
        ( data == lltData )
        &&
        ( type == lltType )
      )
   {
      lltRep ++ ;
      return;
   }
   lltBuf[lltCur].addr = lltAddr;
   lltBuf[lltCur].data = lltData;
   lltBuf[lltCur].type = lltType;
   lltBuf[lltCur].rep = lltRep > 255L ? 255 : lltRep;
   lltRep = 0L;
   lltCur ++ ;
   if ( lltCur >= MAX_LLT )
      lltCur = 0;
   lltBuf[lltCur].addr = lltAddr = addr;
   lltBuf[lltCur].data = lltData = data;
   lltBuf[lltCur].type = lltType = type;
   lltBuf[lltCur].rep  = 0;
}

/**************************************************************/

/* clear the low level trace buffer */

void trc_llt_dump0( void )

{

   for ( lltCur = 0; lltCur < MAX_LLT; lltCur ++ )
      lltBuf[lltCur].type = 0;
   lltCur = 0;
}

/**************************************************************/

/* start a dump of the low level trace buffer */

void trc_llt_dump1( void )

{

   trc_llt( 0, 0, TRC_LLT_NONE );
   lltDmpLine = 0;
   lltDmpNdx = lltCur + 1;
   if ( lltDmpNdx >= MAX_LLT )
      lltDmpNdx = 0;
}

/**************************************************************/

/* return one line of the low level trace, */
/* returns NULL at end.                    */

unsigned char * trc_llt_dump2( void )

{
   int ndx;

   lltDmpLine ++ ;
   if ( lltDmpLine == 1 )     /* 1st line is heading */
   {
                        /*0        1         2         3      */
                        /*123456789012345678901234567890123456*/
      strcpy( trcDmpBuf, "<opr> <--register---> <data - note>" );
      return trcDmpBuf;
   }
   while ( 1 )
   {
      if ( lltDmpNdx == lltCur )
         return NULL;
      if ( lltBuf[lltDmpNdx].type != 0 )
         break;
      lltDmpNdx ++ ;
      if ( lltDmpNdx >= MAX_LLT )
         lltDmpNdx = 0;
   }
   ndx = 0;
   while ( type_nm[ndx].typeId )
   {
      if ( lltBuf[lltDmpNdx].type == type_nm[ndx].typeId )
         break;
      ndx ++ ;
   }
   strcpy( trcDmpBuf, type_nm[ ndx ].typeNm );
   if ( lltBuf[lltDmpNdx].type < TRC_LLT_S_CFG )
   {
      strcat( trcDmpBuf, reg_nm[ lltBuf[lltDmpNdx].addr ] );
      if ( lltBuf[lltDmpNdx].addr == CB_DATA )
         strcpy( prtBuf, "-- " );
      else
         sprintf( prtBuf, "%02X ", lltBuf[lltDmpNdx].data );
      strcat( trcDmpBuf, prtBuf );
      if (    ( lltBuf[lltDmpNdx].addr == CB_DC )
           && ( lltBuf[lltDmpNdx].type == TRC_LLT_OUTB )
           && ( lltBuf[lltDmpNdx].data & CB_DC_SRST )
         )
      {
         strcat( trcDmpBuf, "- START COMMAND: " );
         strcat( trcDmpBuf, trc_get_cmd_name( TRC_FLAG_SRST, 0 ) );
      }
      if (    ( lltBuf[lltDmpNdx].addr == CB_CMD )
           && ( lltBuf[lltDmpNdx].type == TRC_LLT_OUTB )
         )
      {
         strcat( trcDmpBuf, "- START COMMAND: " );
         strcat( trcDmpBuf, trc_get_cmd_name( TRC_FLAG_CMD, lltBuf[lltDmpNdx].data ) );
      }
      if (    (    ( lltBuf[lltDmpNdx].addr == CB_STAT )
                || ( lltBuf[lltDmpNdx].addr == CB_ASTAT )
              )
           && ( lltBuf[lltDmpNdx].type == TRC_LLT_INB )
         )
         strcat( trcDmpBuf, trc_get_st_bit_name( lltBuf[lltDmpNdx].data ) );
      if (    ( lltBuf[lltDmpNdx].addr == CB_ERR )
           && ( lltBuf[lltDmpNdx].type == TRC_LLT_INB )
         )
         strcat( trcDmpBuf, trc_get_er_bit_name( lltBuf[lltDmpNdx].data ) );
   }
   if ( lltBuf[lltDmpNdx].type == TRC_LLT_ERROR )
      strcat( trcDmpBuf, trc_get_err_name( lltBuf[lltDmpNdx].data ) );
   if ( ( lltBuf[lltDmpNdx].type == TRC_LLT_DEBUG )
        ||
        ( lltBuf[lltDmpNdx].type == TRC_LLT_P_CMD )
      )
   {
      sprintf( prtBuf, "%02X ", lltBuf[lltDmpNdx].data );
      strcat( trcDmpBuf, prtBuf );
   }
   if ( lltBuf[lltDmpNdx].rep )
   {
      if ( lltBuf[lltDmpNdx].rep == 255 )
         strcpy( prtBuf, "- repeated 255 or more times " );
      else
         sprintf( prtBuf, "- repeated %u more times ", lltBuf[lltDmpNdx].rep );
      strcat( trcDmpBuf, prtBuf );
   }
   lltDmpNdx ++ ;
   if ( lltDmpNdx >= MAX_LLT )
      lltDmpNdx = 0;
   return trcDmpBuf;
}

/* end ataiotrc.c */
