
/* Example1.c -- Some ATA command examples */

#if 0

   Some simple instructions for those of you that would like to
   compile my driver code and try it.

   1) I've always used Borland C (or C++, the driver code is plain C
   code).  One file contains a few lines of embedded ASM code.

   2) I've always used small memory model but I think the driver code
   will work with any memory mode.

   3) Here is a very small program that shows how to use the driver
   code to issue some ATA commands.

#endif

/*begin*/

#include <stdio.h>
#include <stdlib.h>
#include <dos.h>

#include "ataio.h"

unsigned char * devTypeStr[]
       = { "no device found", "unknown type", "ATA", "ATAPI" };

unsigned char buffer[4096];
unsigned char far * bufferPtr;

/************************************************/

/* a little function to display all the error */
/* and trace information from the driver      */

void ShowAll( void );

void ShowAll( void )

{
   unsigned char * cp;

   printf( "ERROR !\n" );

   /* display the command error information */
   trc_err_dump1();           /* start */
   while ( 1 )
   {
      cp = trc_err_dump2();   /* get and display a line */
      if ( cp == NULL )
         break;
      printf( "* %s\n", cp );
   }

   /* display the command history */
   trc_cht_dump1();           /* start */
   while ( 1 )
   {
      cp = trc_cht_dump2();   /* get and display a line */
      if ( cp == NULL )
         break;
      printf( "* %s\n", cp );
   }

   /* display the low level trace */
   trc_llt_dump1();           /* start */
   while ( 1 )
   {
      cp = trc_llt_dump2();   /* get and display a line */
      if ( cp == NULL )
         break;
      printf( "* %s\n", cp );
   }

   /* now clear the command history and low level traces */
   trc_cht_dump0();     /* zero the command history */
   trc_llt_dump0();     /* zero the low level trace */
}

/************************************************/

int main( int ac, char * av[] )
{
   int dev = 0;
   int numDev;
   int rc;

   /* initialize far pointer to the I/O buffer */
   bufferPtr = (unsigned char far *) buffer;

   /* 1) must tell the driver what the I/O port addresses. */
   /*    note that the driver also supports all the PCMCIA */
   /*    PC Card modes (I/O and memory). */
   pio_set_iobase_addr( 0x1f0, 0x3f0 );
   printf( "\nUsing I/O base addresses %03X and %03X.\n", 0x1f0, 0x3f0 );

   /* 2) find out what devices are present -- this is the step */
   /* many driver writers ignore.  You really can't just do */
   /* resets and commands without first knowing what is out there. */
   /* Even if you don't care the driver does care. */
   numDev = reg_config();
   printf( "\nFound %d devices on this ATA interface:\n", numDev );
   printf( "   Device 0 type is %s.\n", devTypeStr[ reg_config_info[0] ] );
   printf( "   Device 1 type is %s.\n", devTypeStr[ reg_config_info[1] ] );

   /* that's the basics, now do some stuff in polling mode */

   /* do an ATA soft reset (SRST) and return the command block
   /* regs for device 0 in struct reg_cmd_info */
   rc = reg_reset( 0, dev );
   if ( rc )
      ShowAll();

   /* do a seek command in CHS mode to CHS=100,2,5 */
   rc = reg_non_data( dev, CMD_SEEK, 0, 0, 100, 2, 5 );
   if ( rc )
      ShowAll();

   /* do a seek command in LBA mode to LBA=5025 */
   rc = reg_non_data_lba( dev, CMD_SEEK, 0, 0, 5025L );
   if ( rc )
      ShowAll();

   /* do an ATA Identify command in CHS mode */
   rc = reg_pio_data_in(
               dev, CMD_IDENTIFY_DEVICE,
               0, 0,
               0, 0, 0,
               FP_SEG( bufferPtr ), FP_OFF( bufferPtr ),
               1, 0
               );
   if ( rc )
      ShowAll();

   /* do an ATA Identify command in LBA mode */
   rc = reg_pio_data_in_lba(
               dev, CMD_IDENTIFY_DEVICE,
               0, 0,
               0L,
               FP_SEG( bufferPtr ), FP_OFF( bufferPtr ),
               1, 0
               );
   if ( rc )
      ShowAll();

   /* OK, so you want to try using interrupts... */
   /* first you must enable the appropriate irq */
   /* HOWEVER, if call this function you MUST make */
   /* sure your program does not terminate without */
   /* calling int_disable() !!! */
   rc = int_enable_irq( 14 );
   if ( rc )
   {
      printf( "Unable to set interrupt mode using IRQ 14 !\n" );
      printf( " Polling mode remains in effect.\n" );
   }

   /* do a seek command in CHS mode to CHS=100,2,5 */
   rc = reg_non_data( dev, CMD_SEEK, 0, 0, 100, 2, 5 );
   if ( rc )
      ShowAll();

   /* do a seek command in LBA mode to LBA=5025 */
   rc = reg_non_data_lba( dev, CMD_SEEK, 0, 0, 5025L );
   if ( rc )
      ShowAll();

   /* do an ATA Identify command in CHS mode */
   rc = reg_pio_data_in(
               dev, CMD_IDENTIFY_DEVICE,
               0, 0,
               0, 0, 0,
               FP_SEG( bufferPtr ), FP_OFF( bufferPtr ),
               1, 0
               );
   if ( rc )
      ShowAll();

   /* do an ATA Identify command in LBA mode */
   rc = reg_pio_data_in_lba(
               dev, CMD_IDENTIFY_DEVICE,
               0, 0,
               0L,
               FP_SEG( bufferPtr ), FP_OFF( bufferPtr ),
               1, 0
               );
   if ( rc )
      ShowAll();

   /* disable intrq -- you MUST do this if you */
   /* called int_enable_irq() !!! */
   int_disable_irq();

   return 0;
}

/* end example1.c */
